@@ -86,6 +86,21 @@
 						bool representsReal(const MathStructure &vargs, bool allow_units = false) const;\
 						bool representsNonComplex(const MathStructure &vargs, bool allow_units = false) const;\
 					};
+					
+
+#define DECLARE_BUILTIN_FUNCTION_R3(x)	class x : public MathFunction { \
+					  public: \
+						int calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo);  \
+						x(); \
+						x(const x *function) {set(function);} \
+						ExpressionItem *copy() const {return new x(this);} \
+						bool representsNumber(const MathStructure &vargs, bool allow_units = false) const;\
+						bool representsReal(const MathStructure &vargs, bool allow_units = false) const;\
+						bool representsNonComplex(const MathStructure &vargs, bool allow_units = false) const;\
+						bool representsComplex(const MathStructure &vargs, bool allow_units = false) const;\
+						bool representsNonZero(const MathStructure &vargs, bool allow_units = false) const;\
+					};
+										
 #define DECLARE_BUILTIN_FUNCTION_R1(x)	class x : public MathFunction { \
 					  public: \
 						int calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo);  \
@@ -120,6 +135,9 @@ DECLARE_BUILTIN_FUNCTION(PermanentFunction)
 DECLARE_BUILTIN_FUNCTION(AdjointFunction)
 DECLARE_BUILTIN_FUNCTION(CofactorFunction)
 DECLARE_BUILTIN_FUNCTION(InverseFunction)
+DECLARE_BUILTIN_FUNCTION(MagnitudeFunction)
+DECLARE_BUILTIN_FUNCTION(HadamardFunction)
+DECLARE_BUILTIN_FUNCTION(EntrywiseFunction)
 
 DECLARE_BUILTIN_FUNCTION_R(FactorialFunction)
 DECLARE_BUILTIN_FUNCTION_R(DoubleFactorialFunction)
@@ -163,7 +181,8 @@ DECLARE_BUILTIN_FUNCTION_R(ReFunction)
 DECLARE_BUILTIN_FUNCTION_R(ImFunction)
 DECLARE_BUILTIN_FUNCTION_R2(ArgFunction)
 
-DECLARE_BUILTIN_FUNCTION(IntervalFunction)
+DECLARE_BUILTIN_FUNCTION_R(IntervalFunction)
+DECLARE_BUILTIN_FUNCTION(UncertaintyFunction)
 
 DECLARE_BUILTIN_FUNCTION_R(SqrtFunction)
 DECLARE_BUILTIN_FUNCTION(CbrtFunction)
@@ -175,7 +194,7 @@ DECLARE_BUILTIN_FUNCTION(ExpFunction)
 DECLARE_BUILTIN_FUNCTION_R(LogFunction)
 DECLARE_BUILTIN_FUNCTION(LognFunction)
 
-DECLARE_BUILTIN_FUNCTION(LambertWFunction)
+DECLARE_BUILTIN_FUNCTION_R3(LambertWFunction)
 
 DECLARE_BUILTIN_FUNCTION_R2(SinFunction)
 DECLARE_BUILTIN_FUNCTION_R2(CosFunction)
@@ -211,6 +230,7 @@ DECLARE_BUILTIN_FUNCTION(ModeFunction)
 DECLARE_BUILTIN_FUNCTION_RPI(RandFunction)
 
 DECLARE_BUILTIN_FUNCTION(DateFunction)
+DECLARE_BUILTIN_FUNCTION(DateTimeFunction)
 DECLARE_BUILTIN_FUNCTION(TimeValueFunction)
 DECLARE_BUILTIN_FUNCTION(TimestampFunction)
 DECLARE_BUILTIN_FUNCTION(TimestampToDateFunction)
@@ -227,6 +247,9 @@ DECLARE_BUILTIN_FUNCTION(AddDaysFunction)
 DECLARE_BUILTIN_FUNCTION(AddMonthsFunction)
 DECLARE_BUILTIN_FUNCTION(AddYearsFunction)
 
+DECLARE_BUILTIN_FUNCTION(LunarPhaseFunction)
+DECLARE_BUILTIN_FUNCTION(NextLunarPhaseFunction)
+
 DECLARE_BUILTIN_FUNCTION(BinFunction)
 DECLARE_BUILTIN_FUNCTION(OctFunction)
 DECLARE_BUILTIN_FUNCTION(HexFunction)
